@@ -14,7 +14,7 @@ import warnings
 from dataclasses import dataclass
 from datetime import datetime, timezone
 from pathlib import Path
-from typing import TYPE_CHECKING, Callable, NamedTuple
+from typing import TYPE_CHECKING, NamedTuple
 
 from packaging.version import Version
 
@@ -51,10 +51,25 @@ def _subprocess_call(
         env.update(extra_env)
     if isinstance(cmd, str):
         cmd = shlex.split(cmd)
-    proc = subprocess.Popen(
-        cmd, cwd=cwd, env=env, stdout=subprocess.PIPE, stderr=subprocess.PIPE
-    )
-    out, err = proc.communicate()
+    try:
+        proc = subprocess.Popen(
+            cmd, cwd=cwd, env=env, stdout=subprocess.PIPE, stderr=subprocess.PIPE
+        )
+        out, err = proc.communicate()
+    except NotADirectoryError as e:
+        if hasattr(e, "winerror") and e.winerror == 267:
+            # https://github.com/pdm-project/pdm-backend/issues/197
+            # More helpful diagnostic.
+            import textwrap
+
+            err_msg = f"""\
+                       The above error is most likely caused by an unsupported
+                       MSYS2 git binary at {cmd[0]}. Please point your PATH to
+                       a different git binary such as Git For Windows.
+                       """
+            raise subprocess.SubprocessError(textwrap.dedent(err_msg)) from e
+        else:
+            raise e
     return (
         proc.returncode,
         out.decode("utf-8", "surrogateescape").strip(),
@@ -219,28 +234,15 @@ def hg_get_graph_distance(root: StrPath, tag: str | None) -> int:
 
 def _hg_tagdist_normalize_tagcommit(
     config: Config,
-    root: StrPath,
     tag: str,
     dist: int,
     node: str,
     branch: str,
     dirty: bool,
 ) -> SCMVersion:
-    # Detect changes since the specified tag
-    if tag != "0.0":
-        _, commits, _ = _subprocess_call(
-            ["hg", "log", "-r", get_distance_revset(tag), "--template", "{node|short}"],
-            root,
-        )
-    else:
-        commits = "True"
-
-    if commits or dirty:
-        return meta(
-            config, tag, distance=dist or None, node=node, dirty=dirty, branch=branch
-        )
-    else:
-        return meta(config, tag)
+    return meta(
+        config, tag, distance=dist or None, node=node, dirty=dirty, branch=branch
+    )
 
 
 def guess_next_version(tag_version: Version) -> str:
@@ -266,7 +268,7 @@ def _bump_regex(version: str) -> str:
     match = re.match(r"(.*?)(\d+)$", version)
     assert match is not None
     prefix, tail = match.groups()
-    return "%s%d" % (prefix, int(tail) + 1)
+    return f"{prefix}{int(tail) + 1}"
 
 
 def hg_parse_version(root: StrPath, config: Config) -> SCMVersion | None:
@@ -305,13 +307,13 @@ def hg_parse_version(root: StrPath, config: Config) -> SCMVersion | None:
         if tag is None:
             tag = "0.0"
         return _hg_tagdist_normalize_tagcommit(
-            config, root, tag, dist, node, branch, dirty=dirty
+            config, tag, dist, node, branch, dirty=dirty
         )
     except ValueError:
         return None  # unpacking failed, old hg
 
 
-def format_version(version: SCMVersion) -> str:
+def default_version_formatter(version: SCMVersion) -> str:
     if version.distance is None:
         main_version = str(version.version)
     else:
@@ -330,12 +332,8 @@ def format_version(version: SCMVersion) -> str:
 
 
 def get_version_from_scm(
-    root: str | Path,
-    *,
-    tag_regex: str | None = None,
-    tag_filter: str | None = None,
-    version_formatter: Callable[[SCMVersion], str] | None = None,
-) -> str | None:
+    root: str | Path, *, tag_regex: str | None = None, tag_filter: str | None = None
+) -> SCMVersion | None:
     config = Config(
         tag_regex=re.compile(tag_regex) if tag_regex else DEFAULT_TAG_REGEX,
         tag_filter=tag_filter,
@@ -343,7 +341,5 @@ def get_version_from_scm(
     for func in (git_parse_version, hg_parse_version):
         version = func(root, config)
         if version:
-            if version_formatter is None:
-                version_formatter = format_version
-            return version_formatter(version)
+            return version
     return None
